<?php
/**
 * @package     BreezingForms
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
 */
defined('_JEXEC') or die('Direct Access to this location is not allowed.');

use \Joomla\CMS\Filesystem\File;

class BFFactory {

    private static $dbo = null;

    public static function getDbo(){

        if(static::$dbo == null){

            static::$dbo = new BFDbo();
        }

        return static::$dbo;
    }

}

class BFFile extends File {

    public static function read($file){

        return file_get_contents($file);
    }
}

class BFDbo  {

    private $errNo = 0;
    private $errMsg = '';
    private $dbo = null;

    function __construct()
    {
        $this->dbo = JFactory::getDbo();
    }

    public function setQuery($query, $offset = 0, $limit = 0){

        try{

            $this->dbo->setQuery($query, $offset = 0, $limit = 0);

        }catch(Exception $e){

        }
    }

    public function loadObjectList(){

        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->loadObjectList();

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return array();
    }

    public function loadObject($class = 'stdClass')
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->loadObject($class);

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return null;
    }

    public function loadColumn($offset = 0)
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->loadColumn($offset);

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return null;
    }

    public function loadAssocList($key = null, $column = null)
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->loadAssocList($key, $column);

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();

        }

        return null;
    }

    public function loadAssoc()
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->loadAssoc();

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return null;
    }

    public function execute()
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->execute();

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return false;
    }

    public function updateObject($table, $object, $key, $nulls = false)
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->updateObject($table,$object, $key, $nulls);

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return false;
    }

    public function insertObject($table, $object, $key = null)
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try{

            return $this->dbo->insertObject($table, $object, $key);

        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return false;
    }

    public function quote($query, $esc = true)
    {
        return $this->dbo->quote($query, $esc);
    }

    public function getQuery($new = false)
    {
        return $this->dbo->getQuery($new);
    }

    public function getPrefix()
    {
        return $this->dbo->getPrefix();
    }

    public function getNullDate()
    {
        return $this->dbo->getNullDate();
    }

    public function getNumRows()
    {
        return $this->dbo->getNumRows();
    }

    public function getCount()
    {
        return $this->dbo->getCount();
    }

    public function getConnection()
    {
        return $this->dbo->getConnection();
    }

    public function getAffectedRows()
    {
        return $this->dbo->getAffectedRows();
    }

    public function getTableColumns($table, $typeOnly = true)
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try {
            return $this->dbo->getTableColumns($table, $typeOnly);
        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return array();
    }

    public function getTableList()
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try {
            return $this->dbo->getTableList();
        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return array();
    }

    public function loadResult()
    {
        $this->errNo = 0;
        $this->errMsg = '';

        try {
            return $this->dbo->loadResult();
        }catch(Error $e){

            $this->errNo = $e->getCode();
            $this->errMsg = $e->getMessage();
        }

        return null;
    }

    public function getErrorNum()
    {
        return $this->errNo;
    }

    public function getErrorMsg()
    {
        return $this->errMsg;
    }

    public function insertid(){

        return $this->dbo->insertid();
    }
}



if(!defined('DS')){
    define('DS', DIRECTORY_SEPARATOR);
}

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

class com_breezingformsInstallerScript
{
    /**
     * method to install the component
     *
     * @return void
     */
    function install($parent)
    {
        // adjust component menu
        jimport('joomla.version');
        $version = new JVersion();

        if (version_compare($version->getShortVersion(), '3.0', '>=')) {

            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'BreezingForms' " .
                "where `link`='index.php?option=com_breezingforms'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Manage Records', img='components/com_breezingforms/images/js/ThemeOffice/checkin.png' " .
                "where `link`='index.php?option=com_breezingforms&act=managerecs'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Manage Backend Menus', img='components/com_breezingforms/images/js/ThemeOffice/mainmenu.png' " .
                "where `link`='index.php?option=com_breezingforms&act=managemenus'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Manage Forms', img='components/com_breezingforms/images/js/ThemeOffice/content.png' " .
                "where `link`='index.php?option=com_breezingforms&act=manageforms'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Manage Scripts', img='components/com_breezingforms/images/js/ThemeOffice/controlpanel.png' " .
                "where `link`='index.php?option=com_breezingforms&act=managescripts'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Manage Pieces', img='components/com_breezingforms/images/js/ThemeOffice/controlpanel.png' " .
                "where `link`='index.php?option=com_breezingforms&act=managepieces'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Integrator', img='components/com_breezingforms/images/js/ThemeOffice/controlpanel.png' " .
                "where `link`='index.php?option=com_breezingforms&act=integrate'"
            );
            BFFactory::getDbo()->execute();
            BFFactory::getDbo()->setQuery(
                "update #__menu set `alias` = 'Configuration', img='components/com_breezingforms/images/js/ThemeOffice/config.png' " .
                "where `link`='index.php?option=com_breezingforms&act=configuration'"
            );
            BFFactory::getDbo()->execute();
        }
    }

    /**
     * method to update the component
     *
     * @return void
     */
    function update($parent)
    {
        $db = BFFactory::getDbo();
        $tables = self::getTableFields( BFFactory::getDbo()->getTableList() );

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_records']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_records']['opted'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_records` ADD `opted` TINYINT(1) NOT NULL DEFAULT '0' AFTER `paypal_download_tries`, ADD INDEX (`opted`)" );
                $db->execute();
            }
        }

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_records']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_records']['opt_ip'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_records` ADD `opt_ip` VARCHAR(255) NOT NULL DEFAULT '' AFTER `opted`, ADD INDEX (`opt_ip`)" );
                $db->execute();
            }
        }

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_records']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_records']['opt_date'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_records` ADD `opt_date` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00' AFTER `opt_ip`, ADD INDEX (`opt_date`)" );
                $db->execute();
            }
        }

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_records']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_records']['opt_token'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_records` ADD `opt_token` VARCHAR(255) NOT NULL DEFAULT '' AFTER `opt_date`, ADD INDEX (`opt_token`)" );
                $db->execute();
            }
        }

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_forms']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_forms']['double_opt'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_forms` ADD `double_opt` TINYINT(1) NOT NULL DEFAULT '0' AFTER `filter_state`, ADD INDEX (`double_opt`)" );
                $db->execute();
            }
        }

        if(isset($tables[BFFactory::getDbo()->getPrefix().'facileforms_forms']))
        {
            if ( ! isset( $tables[BFFactory::getDbo()->getPrefix() . 'facileforms_forms']['opt_mail'] ) )
            {
                $db->setQuery( "ALTER TABLE `#__facileforms_forms` ADD `opt_mail` VARCHAR(128) NOT NULL DEFAULT '' AFTER `double_opt`, ADD INDEX (`opt_mail`)" );
                $db->execute();
            }
        }
    }

    /**
     * method to uninstall the component
     *
     * @return void
     */
    function uninstall($parent)
    {

        jimport('joomla.filesystem.file');
        jimport('joomla.version');

        $db = BFFactory::getDbo();

        $plugins = $this->getPlugins();

        $installer = new JInstaller();

        foreach($plugins As $folder => $subplugs){

            foreach($subplugs As $plugin){

                $db->setQuery('SELECT `extension_id` FROM #__extensions WHERE `type` = "plugin" AND `element` = "'.$plugin.'" AND `folder` = "'.$folder.'"');

                $id = $db->loadResult();

                if($id)
                {
                    $installer->uninstall('plugin',$id,1);
                }
            }
        }

        $version = new JVersion();

        if(version_compare($version->getShortVersion(), '3.0', '>=')){
            $db = BFFactory::getDbo();
            $db->setQuery("Delete From #__menu Where `link` Like 'index.php?option=com_breezingforms&act=%'");
            $db->execute();
            $db->setQuery("Delete From #__menu Where `alias` Like 'BreezingForms' And `path` Like 'breezingforms'");
            $db->execute();
        }

        if(BFFile::exists(JPATH_SITE.DS.'media'.DS.'breezingforms'.DS.'facileforms.config.php')){
            BFFile::delete(JPATH_SITE.DS.'media'.DS.'breezingforms'.DS.'facileforms.config.php');
        }

        if (BFFile::exists(JPATH_SITE . "/components/com_sh404sef/sef_ext/com_breezingforms.php")){
            BFFile::delete(JPATH_SITE . "/components/com_sh404sef/sef_ext/com_breezingforms.php");
        }

        if(BFFile::exists(JPATH_SITE . '/ff_secimage.php'))BFFile::delete( JPATH_SITE . '/ff_secimage.php');
        if(BFFile::exists(JPATH_SITE . '/templates/system/ff_secimage.php'))BFFile::delete( JPATH_SITE . '/templates/system/ff_secimage.php');
        if(BFFile::exists(JPATH_SITE . "/administrator/components/com_joomfish/contentelements/breezingforms_elements.xml"))BFFile::delete( JPATH_SITE . "/administrator/components/com_joomfish/contentelements/breezingforms_elements.xml");
        if(BFFile::exists(JPATH_SITE . "/administrator/components/com_joomfish/contentelements/translationFformFilter.php"))BFFile::delete( JPATH_SITE . "/administrator/components/com_joomfish/contentelements/translationFformFilter.php");
        if(BFFile::exists(JPATH_SITE . "/administrator/components/com_joomfish/contentelements/translationFformoptions_emptyFilter.php"))BFFile::delete( JPATH_SITE . "/administrator/components/com_joomfish/contentelements/translationFformoptions_emptyFilter.php");


        $db = BFFactory::getDbo();
        $db->setQuery("Select id From `#__menu` Where `alias` = 'root'");
        if(!$db->loadResult()){
            $db->setQuery("INSERT INTO `#__menu` VALUES(1, '', 'Menu_Item_Root', 'root', '', '', '', '', 1, 0, 0, 0, 0, '0000-00-00 00:00:00', 0, 0, '', 0, '', 0, ( Select mlftrgt From (Select max(mlft.rgt)+1 As mlftrgt From #__menu As mlft) As tbone ), 0, '*', 0)");
            $db->execute();
        }
    }

    /**
     * method to run before an install/update/uninstall method
     *
     * @return void
     */
    function preflight($type, $parent)
    {
        $db = BFFactory::getDbo();
        $db->setQuery("Select id From `#__menu` Where `alias` = 'root'");
        if(!$db->loadResult()){
            $db->setQuery("INSERT INTO `#__menu` VALUES(1, '', 'Menu_Item_Root', 'root', '', '', '', '', 1, 0, 0, 0, 0, '0000-00-00 00:00:00', 0, 0, '', 0, '', 0, ( Select mlftrgt From (Select max(mlft.rgt)+1 As mlftrgt From #__menu As mlft) As tbone ), 0, '*', 0)");
            $db->execute();
        }
    }

    /**
     * method to run after an install/update/uninstall method
     *
     * @return void
     */
    function postflight($type, $parent)
    {
        $db = BFFactory::getDbo();

        $plugins = $this->getPlugins();


        $base_path = JPATH_SITE . DS . 'administrator' . DS . 'components' . DS . 'com_breezingforms' . DS . 'plugins';

        $folders = JFolder::folders($base_path);

        $installer = new JInstaller();

        foreach( $folders As $folder ){
            $installer->install( $base_path . DS . $folder );
        }

        foreach($plugins As $folder => $subplugs){
            foreach($subplugs As $plugin){
                $db->setQuery('Update #__extensions Set `enabled` = 1 WHERE `type` = "plugin" AND `element` = "'.$plugin.'" AND `folder` = "'.$folder.'"');
                $db->execute();
            }
        }

        $db->setQuery("Select id From `#__menu` Where `alias` = 'root'");
        if(!$db->loadResult()){
            $db->setQuery("INSERT INTO `#__menu` VALUES(1, '', 'Menu_Item_Root', 'root', '', '', '', '', 1, 0, 0, 0, 0, '0000-00-00 00:00:00', 0, 0, '', 0, '', 0, ( Select mlftrgt From (Select max(mlft.rgt)+1 As mlftrgt From #__menu As mlft) As tbone ), 0, '*', 0)");
            $db->execute();
        }

        $db->setQuery("Select update_site_id From #__update_sites Where `name` = 'BreezingForms Lite' And `type` = 'extension'");
        $site_id = $db->loadResult();

        if( $site_id ){

            $db->setQuery("Delete From #__update_sites Where update_site_id = " . $db->quote($site_id));
            $db->execute();
            $db->setQuery("Delete From #__update_sites_extensions Where update_site_id = " . $db->quote($site_id));
            $db->execute();
            $db->setQuery("Delete From #__updates Where update_site_id = " . $db->quote($site_id));
            $db->execute();
        }
    }

    function getPlugins(){
        $plugins = array();
        $plugins['system'] = array();
        $plugins['system'][] = 'sysbreezingforms';
        return $plugins;
    }

    public static function getTableFields($tables, $typeOnly = true)
    {

        $results = array();

        settype($tables, 'array');

        foreach ($tables as $table)
        {
            $results[$table] = BFFactory::getDbo()->getTableColumns($table, $typeOnly);
        }

        return $results;
    }
}

